(load "support/genhtml.scm")
(load "support/athena-defs.scm")

(define (phd-thesis when)
  (TEXT 
   "Ph. D. Thesis, MIT Department of Electrical Engineering and Computer Science."
   (BR)
   when "."))

(define (masters-thesis when)
  (TEXT
   "Master's Thesis, MIT Department of Electrical Engineering and Computer Science."
   (BR)
   when "."))

(define (by authors)
  (case (length authors)
    ((1) (car authors))
    ((2) (TEXT (car authors) " and " (cadr authors)))
    (else
     (let ((butlast (reverse (cdr (reverse authors))))
	   (last-guy (car (reverse authors))))
       (TEXT
	(apply $SEQUENTIALLY
	       (map (lambda (guy) (TEXT guy ", ")) butlast))
	" and "
	(TEXT last-guy))))))

(define some-html
  (lambda (url remarks)
    ($SEQUENTIALLY
     (if (not (null? remarks))
	 ($SEQUENTIALLY
	  (apply $SEQUENTIALLY (map TEXT remarks))
	  (BR))
	 NIL)
     (if url
	 (simple-link url "[Full text]")
	 NIL))))

(define abstract-link
  (lambda (title authors abstract abstract-text url remarks)
    (with-output-to-file (string-append "abstracts/" abstract
					".html") 
      (lambda ()
	(print-html (athena-page "Abstract" "../../" 
			       ($SEQUENTIALLY
				(A:NAME "afterheading" (TEXT ""))
				(HR)
				(CENTER (H3 (TEXT title)))
				(CENTER (TEXT "by " (by authors) "."))
				(P (CENTER (some-html url remarks)))
				(HR)
				(P (CENTER (H3 (TEXT "Abstract"))))
				abstract-text)))))
    ($SEQUENTIALLY 
     (TEXT " ")
     (simple-link (string-append "abstracts/"
				 abstract ".html#afterheading")
		  "Abstract")
     (BR))))
     
(define paper 
  (let ((count 0))
    (lambda (title authors abstract-text url . remarks)
      (set! count (+ count 1))
      (let ((abstract
	     (string-append "abstract" (number->string count))))
	(LI 
	 ($SEQUENTIALLY
	  (STRONG (TEXT title))
	  (BR)
	  (TEXT "by " (by authors) ".")
	  (BR)
	  (some-html url remarks)
	  (if abstract-text
	      (abstract-link title authors abstract abstract-text url remarks)
	      NIL)
	  (P NIL) ))))))

(define papers-list
  (UL

   (paper "Everyone Loves File: File Storage Service (FSS) in Oracle
Cloud Infrastructure"
	  (list
           "Bradley C. Kuszmaul"
           Matteo-Frigo
           "Justin Mazzola Paluska"
           "Alexander (Sasha) Sandler")
	  (P (TEXT
	      "File Storage Service (FSS) is an elastic filesystem
provided as a managed NFS service in Oracle Cloud
Infrastructure. Using a pipelined Paxos implementation, we implemented
a scalable block store that provides linearizable multipage
limited-size transactions. On top of the block store, we built a
scalable B-tree that provides linearizable multikey limited-size
transactions. By using self-validating B-tree nodes and performing all
B-tree housekeeping operations as separate transactions, each key in a
B-tree transaction requires only one page in the underlying block
transaction. The B-tree holds the filesystem metadata. The filesystem
provides snapshots by using versioned key-value pairs. The entire
system is programmed using a nonblocking lock-free programming
style. The presentation servers maintain no persistent local state,
with any state kept in the B-tree, making it easy to scale up and
failover the presentation servers. We use a non-scalable
Paxos-replicated hash table to store configuration information
required to bootstrap the system. The system throughput can be
predicted by comparing an estimate of the network bandwidth needed for
replication to the network bandwidth provided by the hardware. Latency
on an unloaded system is about 4 times higher than a Linux NFS server
backed by NVMe, reflecting the cost of replication."))
          (on-paperdir "usenix-atc19-fss-paper.pdf")
	  (CITE (TEXT "Proceedings of the 2019 USENIX Annual Technical Conference."))
          )

   (paper "Reducers and Other Cilk++ Hyperobjects"
	  (list Matteo-Frigo "Pablo Halpern" cel "Stephen Lewin-Berlin")
	  (P (TEXT
	      "This paper introduces hyperobjects, a linguistic mechanism
  that allows different branches of a multithreaded program to
  maintain coordinated local views of the same nonlocal variable.  We
  have identified three kinds of hyperobjects that seem to be useful
  --- reducers, holders, and splitters --- and we
  have implemented reducers and holders in Cilk++, a set of extensions
  to the C++ programming language that enables multicore programming
  in the style of MIT Cilk.  We analyze a randomized locking
  methodology for reducers and show that a work-stealing scheduler can
  support reducers without incurring significant overhead."))
          (on-paperdir "hyper.pdf")
	  (CITE (TEXT "SPAA '09 Proceedings of the twenty-first annual symposium on Parallelism in algorithms and architectures, Pages 79--90."))
	  (BR)
	  (TEXT " SPAA 2009 Best Paper Award."))

   (paper "Parallel Sparse Matrix-Vector and 
Matrix-Transpose-Vector Multiplication Using Compressed Sparse Blocks"
	  (list Aydin-Buluc Jeremy-Fineman Matteo-Frigo John-Gilbert cel)
	  #f
          (on-paperdir "csb.pdf")
	  (CITE (TEXT "SPAA '09 Proceedings of the twenty-first annual symposium on Parallelism in algorithms and architectures, Pages 233--244")))

   (paper "The Cache Complexity of Multithreaded Cache Oblivious Algorithms"
	  (list Matteo-Frigo Volker-Strumpen)
	  (P (TEXT
	      "We present a technique for analyzing the number of cache misses
incurred by multithreaded cache oblivious algorithms on an idealized
parallel machine in which each processor has a private cache.  We
specialize this technique to computations executed by the Cilk
work-stealing scheduler on a machine with dag-consistent shared
memory.  We show that a multithreaded cache oblivious matrix
multiplication incurs $O(n^3/\\sqrt{Z} + (P n)^{1/3}
n^2)$ cache misses when executed by the Cilk scheduler on a machine
with~$P$ processors, each with a cache of size~$Z$,
with high probability. This bound is tighter than previously
published bounds.  We also present a new multithreaded cache
oblivious algorithm for 1D stencil computations incurring
$O(n^2/Z + n + \\sqrt{P n^{3+\\epsilon}})$ cache misses
with high probability, one for Gaussian elimination and back
substitution, and one for the length computation part of the longest
common subsequence problem incurring $O(n^2/Z +
\\sqrt{P n^{3.58}})$ cache misses with high probability."))
          (on-paperdir "tocs08.pdf")
	  (CITE (TEXT "Theory of Computing Systems August 2009, Volume
45, Issue 2, pp 203–-233."))
	  (BR)
	  (TEXT " An earlier version of this paper appears in the
Proceedings of the 18th ACM Symposium on Parallelism in Algorithms and
Architectures (SPAA'06)."))
   
   (paper "A modified split-radix FFT with fewer arithmetic operations"
	  (list Steven-Johnson Matteo-Frigo)
	  (P (TEXT
"Recent results by Van Buskirk et al. have broken
the record set by Yavne in 1968 for the lowest exact count of
real additions and multiplications to compute a power-of-two
discrete Fourier transform (DFT). Here, we present a simple
recursive modification of the split-radix algorithm that computes
the DFT with asymptotically about 6% fewer operations than
Yavne, matching the count achieved by Van Buskirk's program generation
framework. We also discuss the application of our
algorithm to real-data and real-symmetric (discrete cosine) transforms,
where we are again able to achieve lower arithmetic counts
than previously published algorithms."))
          (on-paperdir "newsplit-pub.pdf")
	  (CITE (TEXT "IEEE Transactions on Signal Processing, volume 55, number 1, Jan 2007, pages 111-119")))

   (paper "Software Engineering Aspects of Cache Oblivious Stencil Computations"
	  (list  Volker-Strumpen Matteo-Frigo)
	  (P (TEXT
	      "We discuss the design of cache oblivious stencil computations for
finite-difference methods in 1-dimensional space and time.  We employ
our cache oblivious spacetime traversal [ICS'05] to obtain several
fully cache oblivious algorithms.  We focus on the practical aspects
of the software engineering chores that today's superscalar
architectures impose on high performance programming."))
          (on-paperdir "rc24035.pdf")
	  (CITE (TEXT "IBM Research Report RC24035 (W0608-077)")))

   (paper "Cache Oblivious Stencil Computations"
	  (list Matteo-Frigo Volker-Strumpen)
	  (P (TEXT
"We present a cache oblivious algorithm for stencil computations,
which arise for example in finite-difference methods.  Our algorithm
applies to arbitrary stencils in $n$-dimensional spaces.  On an
``ideal cache'' of size $Z$, our algorithm saves a factor
of $\Theta(Z^{1/n})$ cache misses compared to a naive
algorithm, and it exploits temporal locality optimally throughout the
entire memory hierarchy."))
          (on-paperdir "ics05.pdf")
	  (CITE (TEXT "Proceedings of the 19th ACM International Conference on Supercomputing (ICS05).")))

   (paper "The Design and Implementation of FFTW3"
	  (list Matteo-Frigo Steven-Johnson)
	  (P (TEXT
"FFTW is an implementation of the discrete Fourier transform
 (DFT) that adapts to the hardware in order to maximize performance.
This paper shows that such an approach can yield an implementation
that is competitive with hand-optimized libraries, and describes the
software structure that makes our current FFTW3 version flexible and
adaptive.  We further discuss a new algorithm for real-data DFTs of
prime size, a new way of implementing DFTs by means of
machine-specific ``SIMD'' instructions, and how a special-purpose
compiler can derive optimized implementations of the discrete cosine
and sine transforms automatically from a DFT algorithm."))
	  (on-paperdir "fftw-paper-ieee.ps.gz")
	  (CITE (TEXT "Proceedings of the IEEE, vol. 93, no. 2, pp. 216--231 (2005)")))

   (paper "A Noise-Adaptive Strategy for First-Order Reed-Muller Decoding"
	  (list Jon-Feldman Ibrahim-Abou-Faycal Matteo-Frigo)
	  (P (TEXT 
"We consider the problem of decoding First-Order Reed-Muller codes
efficiently.  We give an algorithm that implicitly adapts to the noise
conditions, runs significantly faster than known maximum-likelihood
algorithms, and yields an error rate that is very close to optimal.
When applied to CCK demodulation (used in the 802.11b standard for
Wireless Local Area Networks), the algorithm runs up to 4 times faster
than a decoder based on the Fast Hadamard Transform, with a loss of at
most 0.2dB in error rate.  We show analytically that the error
rate of our adaptive algorithm is 2^-\Omega(n), where n is the
length of a codeword."))
	  (on-paperdir "reed-muller.ps.gz")
	  (CITE (TEXT "IEEE Semiannual Vehicular Technology Conference, Fall 2002")))

   (paper "A Fast Maximum-Likelihood Decoder for Convolutional Codes"
	  (list Jon-Feldman Ibrahim-Abou-Faycal Matteo-Frigo)
(P (TEXT
  "The " (STRONG (TEXT "lazy Viterbi decoder")) 
  " is a maximum-likelihood decoder for
  block and stream convolutional codes.  For many codes of practical
  interest, under reasonable noise conditions, the lazy decoder is
  much faster than the original Viterbi decoder.  For a code of
  constraint length 6, the lazy algorithm is about 50% faster than
  an optimized implementation of the Viterbi decoder whenever
  SNR > 6dB.  Moreover, while the running time of
  the Viterbi decoder grows exponentially with the constraint length,
  under reasonable noise conditions, the running time of the lazy
  algorithm is essentially independent of the constraint length.  This
  paper introduces the lazy Viterbi decoder and shows how to make it
  efficient in practice."))
	  (on-paperdir "viterbi.ps.gz")
	  (CITE (TEXT "IEEE Semiannual Vehicular Technology Conference, Fall 2002")))


   (paper "Portable High-Performance Programs"
	  (list Matteo-Frigo)
	  ($SEQUENTIALLY
	   (P (TEXT "This dissertation discusses how to write computer
programs that attain both high performance and portability, despite
the fact that current computer systems have different degrees of
parallelism, deep memory hierarchies, and diverse processor
architectures."))
	   (P (TEXT "To cope with parallelism portably in
high-performance programs, we present the" (STRONG (TEXT " Cilk "))
"multithreaded programming system.  In the Cilk-5 system, parallel
programs scale up to run efficiently on multiple processors, but
unlike existing parallel-programming environments, such as MPI and
HPF, Cilk programs ``scale down'' to run on one processor as
efficiently as a comparable C program. The typical cost of spawning a
parallel thread in Cilk-5 is only between 2 and 6 times the cost of a
C function call.  This efficient implementation was guided by the"
(STRONG (TEXT " work-first principle ")) ", which dictates that
scheduling overheads should be borne by the critical path of the
computation and not by the work.  We show how the work-first principle
inspired Cilk's novel ``two-clone'' compilation strategy and its
Dijkstra-like mutual-exclusion protocol for implementing the ready
deque in the work-stealing scheduler."))
(P (TEXT "To cope portably with the memory hierarchy, we
present asymptotically optimal algorithms for rectangular matrix
transpose, FFT, and sorting on computers with multiple levels of
caching.  Unlike previous optimal algorithms, these algorithms are"
(STRONG (TEXT " cache oblivious")) ": no variables dependent on
hardware parameters, such as cache size and cache-line length, need to
be tuned to achieve optimality.  Nevertheless, these algorithms use an
optimal amount of work and move data optimally among multiple levels
of cache.  For a cache with size $Z$ and cache-line length $L$ where
$Z=\\Omega(L" (SUP (TEXT "2")) ")$ the number of cache misses for an
$m\\times n$ matrix transpose is $\\Theta(1+mn/L)$.  The number of cache
misses for either an $n$-point FFT or the sorting of $n$ numbers is
$\\Theta(1+(n/L)(1+\\log_Z n))$.  We also give a $\\Theta(mnp)$-work
algorithm to multiply an $m\\times n$ matrix by an $n\\times p$ matrix
that incurs $\\Theta(1 + (mn+np+mp)/L + mnp/L\\sqrt{Z})$ cache
faults."))
	   (P (TEXT "To attain portability in the face of both
parallelism and the memory hierarchy at the same time, we examine the"
(STRONG (TEXT " location consistency ")) "memory model and the" 
(STRONG (TEXT " Backer ")) "coherence algorithm for maintaining it.  We
prove good asymptotic bounds on the execution time of Cilk programs
that use location-consistent shared memory."))
	   (P (TEXT "To cope with the diversity of processor
architectures, we develop the FFTW self-optimizing program, a portable
C library that computes Fourier transforms.  FFTW is unique in that it
can automatically tune itself to the underlying hardware in order to
achieve high performance.  Through extensive benchmarking, FFTW has
been shown to be typically faster than all other publicly available
FFT software, including codes such as Sun's Performance Library and
IBM's ESSL that are tuned to a specific machine.  Most of the
performance-critical code of FFTW was generated automatically by a
special-purpose compiler written in Objective Caml, which uses
symbolic evaluation and other compiler techniques to produce
``codelets''---optimized sequences of C code that can be assembled
into ``plans'' to compute a Fourier transform.  At runtime, FFTW
measures the execution time of many plans and uses dynamic programming
to select the fastest. Finally, the plan drives a special interpreter
that computes the actual transforms.")))
	  (on-paperdir "frigo-phd-thesis.ps.gz")
	  (phd-thesis "June 1999"))

   (paper "Cache-Oblivious Algorithms"
	  (list Matteo-Frigo cel Harald-Prokop Sridhar-Ramachandran)
	  ($SEQUENTIALLY
	   (P (TEXT "This paper presents asymptotically optimal
algorithms for rectangular matrix transpose, FFT, and sorting on
computers with multiple levels of caching.  Unlike previous optimal
algorithms, these algorithms are <i><b>cache oblivious</b></i>: no
variables dependent on hardware parameters, such as cache size and
cache-line length, need to be tuned to achieve optimality.
Nevertheless, these algorithms use an optimal amount of work and move
data optimally among multiple levels of cache.  For a cache with size
<i>Z</i> and cache-line length <i>L</i> where <i>Z=\\Omega(L^2)</i> the
number of cache misses for an <i>m\\times n</i> matrix transpose is
<i>\\Theta(1+mn/L)</i>.  The number of cache misses for either an
<i>n</i>-point FFT or the sorting of <i>n</i> numbers is
<i>\\Theta(1+(n/L)(1+\\logZ n))</i>.  We also give an
<i>\\Theta(mnp)</i>-work algorithm to multiply an <i>m\\times n</i>
matrix by an <i>n\\times p</i> matrix that incurs <i>\\Theta(1 +
(mn+np+mp)/L + mnp/L\\sqrt{Z})</i> cache faults."))
	   (P (TEXT "We introduce an ``ideal-cache'' model to analyze
our algorithms, and we prove that an optimal cache-oblivious algorithm
designed for two levels of memory is also optimal for multiple levels.
We also prove that any optimal cache-oblivious algorithm is also
optimal in the previously studied HMM and SUMH models.  Algorithms
developed for these earlier models are perforce
<b><i>cache-aware</i></b>: their behavior varies as a function of
hardware-dependent parameters which must be tuned to attain
optimality.  Our cache-oblivious algorithms achieve the same
asymptotic optimality, but without any tuning.")))
	  (on-paperdir "FrigoLePr99.ps.gz")
	  (CITE (TEXT "In the 40th Annual Symposium on Foundations of Computer Science, FOCS '99, 17-18 October, 1999, New York, NY, USA.")))

   (paper "A Fast Fourier Transform Compiler"
	  (list Matteo-Frigo)
	  ($SEQUENTIALLY
	   (P (TEXT "The FFTW library for computing the discrete
Fourier transform (DFT) has gained a wide acceptance in both academia
and industry, because it provides excellent performance on a variety
of machines (even competitive with or faster than equivalent libraries
supplied by vendors).  In FFTW, most of the performance-critical code
was generated automatically by a special-purpose compiler, called
<tt>genfft</tt>, that outputs C code.  Written in Objective Caml,
<tt>genfft</tt> can produce DFT programs for any input length, and it
can specialize the DFT program for the common case where the input
data are real instead of complex.  Unexpectedly, <tt>genfft</tt>
``discovered'' algorithms that were previously unknown, and it was
able to reduce the arithmetic complexity of some other existing
algorithms.  This paper describes the internals of this
special-purpose compiler in some detail, and it argues that a
specialized compiler is a valuable tool.")))
	  (on-paperdir "pldi99.ps.gz")
          (CITE (TEXT "In the ACM SIGPLAN'99 Conference on Programming "
              "Language Design and Implementation (PLDI), Atlanta, GA, USA"))
	  (BR)
          (TEXT "2009 Most Influential PLDI Paper Award (for 1999)"))

    (paper "Computation-Centric Memory Models"
 	  (list Matteo-Frigo "Victor Luchangco")
 	  ($SEQUENTIALLY
 	   (P (TEXT "We present a computation-centric theory of memory
 models.  Unlike traditional processor-centric models,
 computation-centric models focus on the logical dependencies among
 instructions rather than the processor that happens to execute them.
 This theory allows us to define what a memory model is, and to
 investigate abstract properties of memory models.  In particular, we
 focus on constructibility, which is a necessary property of those
 models that can be implemented exactly by an online algorithm.  For a
 nonconstructible model, we show that there is a natural way to define
 the constructible version of that model.  We explore the implications
 of constructibility in the context of dag-consistent memory models,
 which do not require that memory locations be serialized.  The
 strongest dag-consistent model, called NN-dag consistency, is not
 constructible.  However, its constructible version is equivalent to a
 model that we call location consistency, in which each location is
 serialized independently.")))
 	  (on-paperdir "spaa98-memory.ps.gz")
 	  (CITE (TEXT "In the 10th ACM Symposium on Parallel Algorithms and Architectures, June 28-July 2, 1998.")))

    (paper "FFTW: An Adaptive Software Architecture for the FFT"
           (list Matteo-Frigo Steven-Johnson)
           (P (TEXT 
 "FFT literature has been mostly concerned with minimizing the number
  of floating-point operations performed by an algorithm.
  Unfortunately, on present-day microprocessors this measure is far
  less important than it used to be, and interactions with the
  processor pipeline and the memory hierarchy have a larger impact on
  performance.  Consequently, one must know the details of a computer
  architecture in order to design a fast algorithm.  In this paper, we
  propose an adaptive FFT program that tunes the computation
  automatically for any particular hardware.  We compared our program,
  called FFTW, with over 40 implementations of the FFT on 7 machines.
  Our tests show that FFTW's self-optimizing approach usually yields
  significantly better performance than all other publicly available
  software.  FFTW also compares favorably with machine-specific,
  vendor-optimized libraries."))
           (on-paperdir "fftw-paper-icassp.ps.gz")
           (CITE (TEXT "In the Proceedings of the 1998 ICASSP conference (vol. 3, pp. 1381-1384)")))

    (paper "The Weakest Reasonable Memory Model"
 	  (list Matteo-Frigo)
 	  ($SEQUENTIALLY
 	   (P (TEXT "A memory model is some description of how memory
 behaves in a parallel computer system.  While there is consensus that
 sequential consistency [Lamport 79] is the strongest memory model,
 nobody seems to have tried to identify the weakest memory model.  This
 thesis concerns itself with precisely this problem."))
 	   (P (TEXT "We cannot hope to identify the weakest memory
 model unless we specify a minimal set of properties we want it to
 obey.  In this thesis, we identify five such properties: completeness,
 monotonicity, constructibility, nondeterminism confinement, and
 classicality.  Constructibility is especially interesting, because a
 nonconstructible model cannot be implemented exactly, and hence every
 implementation necessarily supports a stronger model.  One
 nonconstructible model is, for example, dag consistency [Blumofe et
 al. 96]."))
 	   (P (TEXT "We argue (with some caveats) that if one wants
 the five properties, then location consistency is the weakest
 reasonable memory model.  In location consistency, every memory
 location is serialized, but different locations may be serialized
 independently.  (Location consistency is sometimes called coherence
 [Hennessy and Patterson 96], and our location consistency is"
 (STRONG (TEXT " not ")) " the model with the same name proposed by [Gao and
 Sarkar 93].)"))
 	   (P (TEXT "We obtain these results within a
 computation-centric theory of memory models, where memory models are
 defined independently of scheduling issues and language semantics.")))
 	  (on-paperdir "frigo-ms-thesis.ps.gz")
 	  (masters-thesis "January 1998"))

   (paper "The Implementation of the Cilk-5 Multithreaded Language"
	  (list Matteo-Frigo cel Keith-Randall)
	  ($SEQUENTIALLY
            (P (TEXT "The fifth release of the multithreaded language
Cilk uses a provably good ``work-stealing'' scheduling algorithm
similar to the first system, but the runtime system has been
completely reengineered.  The efficiency of the new implementation was
aided by a clear strategy that arose from a theoretical analysis of
the scheduling algorithm: concentrate on minimizing overheads that
contribute to the work, even at the expense of overheads that
contribute to the critical path.  Although it may seem
counterintuitive to move overheads from the work of a computation to
its critical path, this ``work-first'' principle has led to a portable
Cilk-5 implementation in which the typical cost of spawning a parallel
thread is only between 2 and 6 times the cost of a C function call on
a variety of contemporary machines.  Many Cilk programs run on one
processor with virtually no degradation compared to equivalent C
programs.  This paper describes how the work-first principle was
exploited in the design of Cilk-5's compiler and its runtime system.
In particular, we present Cilk-5's novel ``two-clone'' compilation
strategy and its Dijkstra-like mutual-exclusion protocol for
implementing the ready deque in the work-stealing scheduler.")))
          (on-paperdir "cilk5.ps.gz")
	  (CITE (TEXT "1998 ACM SIGPLAN Conference on Programming Language"))
	  (BR)
          (TEXT "2008 Most Influential PLDI Paper Award (for 1998)"))

    (paper "The Fastest Fourier Transform in the West"
 	  (list Matteo-Frigo Steven-Johnson)
 	  #f
 	  (on-paperdir "fftw-paper.ps.gz")
 	  "Technical Report MIT-LCS-TR-728, September 1997.")

    (paper "An Analysis of Dag-Consistent Distributed Shared-Memory Algorithms"
 	  (list Bobby-Blumofe Matteo-Frigo Chris-Joerg cel Keith-Randall)
 	  ($SEQUENTIALLY
 	   (P (TEXT "In this paper, we analyze the performance of
 parallel multithreaded algorithms that use dag-consistent distributed
 shared memory.  Specifically, we analyze execution time, page faults,
 and space requirements for multithreaded algorithms executed by a
 work-stealing thread scheduler and the Backer coherence algorithm for
 maintaining dag consistency.  We prove that if the accesses to the
 backing store are random and independent (the Backer algorithm
 actually uses hashing), then the expected execution time of a ``fully
 strict'' multithreaded computation on " (EM (TEXT " P ")) "
 processors, each with an LRU cache of " (EM (TEXT " C ")) " pages, is
 " (EM (TEXT " O(T<sub>1</sub>(C)/P + m C T<sub>inf</sub>) ")) ",
 where " (EM (TEXT " T<sub>1</sub>(C) ")) " is the total work of the
 computation including page faults," (EM (TEXT " T<sub>inf</sub> ")) "
 is its critical-path length excluding page faults, and " (EM (TEXT " m
 ")) " is the minimum page transfer time.  As a corollary to this
 theorem, we show that the expected number of page faults incurred by a
 computation executed on " (EM (TEXT " P ")) " processors, each with an
 LRU cache of " (EM (TEXT " C ")) " pages, is " (EM (TEXT "
 F<sub>1</sub>(C) + O(C P T<sub>inf</sub>) ")) ", where " (EM (TEXT
 " F<sub>1</sub>(C) ")) " is the number of serial page faults.
 Finally, we give simple bounds on the number of page faults and the
 space requirements for ``regular'' divide-and-conquer algorithms.  We
 use these bounds to analyze parallel multithreaded algorithms for
 matrix multiplication and LU-decomposition.")))
 	  (on-paperdir "spaa96.ps.gz")
 	  "In "
 	  (CITE (TEXT "8th Annual ACM Symposium on Parallel Algorithms and Architectures"))
 	  (BR)
 	  (simple-link "http://www.cs.jhu.edu/Conferences/SPAA96/"
 		       "SPAA'96")
 	  ", June 24-26, 1996, Padua, Italy, pp.297-308.")

    (paper "Dag-Consistent Distributed Shared Memory"
 	  (list Bobby-Blumofe Matteo-Frigo Chris-Joerg cel Keith-Randall)
 	  ($SEQUENTIALLY
 	   (P (TEXT "We introduce dag consistency, a relaxed
 consistency model for distributed shared memory which is suitable for
 multithreaded programming.  We have implemented dag consistency in
 software for the Cilk multithreaded runtime system running on a
 Connection Machine CM5.  Our implementation includes a dag-consistent
 distributed cactus stack for storage allocation.  We provide empirical
 evidence of the flexibility and efficiency of dag consistency for
 applications that include blocked matrix multiplication, Strassen's
 matrix multiplication algorithm, and a Barnes-Hut code.  Although Cilk
 schedules the executions of these programs dynamically, their
 performances are competitive with statically scheduled implementations
 in the literature.  We also prove that the number" (EM (TEXT "
 F<sub>P</sub> ")) " of page faults incurred by a user program running
 on " (EM (TEXT " P ")) " processors can be related to the number
 " (EM (TEXT " F<sub>1</sub> ")) " of page faults running serially by
 the formula" (EM (TEXT " F<sub>P</sub> <= F<sub>1</sub> + 2Cs ")) ",
 where " (EM (TEXT " C ")) " is the cache size and " (EM (TEXT " s "))
 " is the number of thread migrations executed by Cilk's scheduler.")))
 	  (on-paperdir "ipps96.ps.gz")
 	  "In "
 	  (CITE (TEXT "10th International Parallel Processing Symposium"))
 	  " (IPPS '96),	April 15-19, 1996, Honolulu, Hawaii, pp. 132-141.")

))

(define page-contents
  ($SEQUENTIALLY
   (section "Papers (co-)authored by Matteo Frigo")
   papers-list))

(define page 
  (athena-page "Papers (co-)authored by Matteo Frigo" "./" page-contents))

(print-html page)
